terraform {
  required_version          = ">= 0.13"
  required_providers {
    aws = {
      source                = "hashicorp/aws"
      version               = "~> 5.0"
    }
  }
}

// provider "null" {}

provider "aws" {
  profile                   = "midot"
  region                    = var.region
}

locals {
  workspace_numbers = {
    cloud1                  = 1
    cloud2                  = 2
    cloud3                  = 3
  }

  workspace_number  = local.workspace_numbers[terraform.workspace] - 1
  config                    = jsondecode(file("${path.module}/configs/${terraform.workspace}.json"))
}

//*******************************************
// Define Availability Zones
//*******************************************
data "aws_availability_zones" "available" {
  state                     = "available"

  filter {
    name                    = "zone-type"
    values                  = ["availability-zone"]
  }
}

//*******************************************
// Define Application Load Balancer
//*******************************************
resource "aws_lb" "alb" {
  name                      = "${terraform.workspace}-lb"
  idle_timeout              = 180
  internal                  = true
  load_balancer_type        = "application"
  security_groups           = [
    aws_security_group.alb_sg.id,
  ]
  subnets                   = var.subnets_id[local.workspace_number]
  
  enable_deletion_protection = true
/*
  access_logs {
    bucket                  = 
    prefix                  = "${terraform.workspace}-alb"
    enabled                 = false
  }
*/
  tags = {
    Name                    = "${terraform.workspace}-lb"
    Environment             = "${terraform.workspace}"
  }
}

//*******************************************
// Define Application LB Security Group
//*******************************************
resource "aws_security_group" "alb_sg" {
  name                      = "${terraform.workspace}-alb-sg"
  description               = "${terraform.workspace} ALB SG"
  vpc_id                    = var.vpc_id

  ingress {
    description             = "Static content from Tomcat"
    from_port               = 80
    to_port                 = 80
    protocol                = "tcp"
    self                    = false
    security_groups         = [
      "sg-aed4e4c7",                                        // WebApache_SG
    ]
  }

  ingress {
    description             = "am"
    from_port               = 8080
    to_port                 = 8080
    protocol                = "tcp"
    self                    = false
    security_groups         = [
      "sg-aed4e4c7",                                        // WebApache_SG
    ]
  }

  ingress {
    description             = "builder from am"
    from_port               = 8081
    to_port                 = 8081
    protocol                = "tcp"
    self                    = false
    security_groups         = [
      "sg-444b282d",                                        // WebJBoss_SG
    ]
  }

  ingress {
    description             = "builder"
    from_port               = 8081
    to_port                 = 8081
    protocol                = "tcp"
    self                    = false
    security_groups         = [
      "sg-aed4e4c7",                                        // WebApache_SG
    ]
  }

  ingress {
    description             = "player-web from am"
    from_port               = 8084
    to_port                 = 8084
    protocol                = "tcp"
    self                    = false
    security_groups         = [
      "sg-444b282d",                                        // WebJBoss_SG
    ]
  }

  ingress {
    description             = "player-web"
    from_port               = 8084
    to_port                 = 8084
    protocol                = "tcp"
    self                    = false
    security_groups         = [
      "sg-aed4e4c7",                                        // WebApache_SG
    ]
  }

  ingress {
    description             = "player from am"
    from_port               = 9080
    to_port                 = 9080
    protocol                = "tcp"
    self                    = false
    security_groups         = [
      "sg-444b282d",                                        // WebJBoss_SG
    ]
  }

  ingress {
    description             = "player"
    from_port               = 9080
    to_port                 = 9080
    protocol                = "tcp"
    self                    = false
    security_groups         = [
      "sg-aed4e4c7",                                        // WebApache_SG
    ]
  }

  egress {
    description             = ""
    from_port               = 0
    to_port                 = 0
    protocol                = -1
    self                    = false
    cidr_blocks             = [
      "0.0.0.0/0"
    ]
  }

  tags                      = {
    Name                    = "${terraform.workspace}-alb-sg"
    Description             = "${terraform.workspace} ALB SG"
  }
}

//*******************************************
// Define Application LB Listeners
//*******************************************
resource "aws_lb_listener" "alb" {
  count                     = length(local.config)

  load_balancer_arn         = aws_lb.alb.arn
  port                      = local.config[count.index].port
  protocol                  = local.config[count.index].protocol

  tags = {
    Name                    = "${terraform.workspace}-alb-${local.config[count.index].port}"
    Description             = "${local.config[count.index]._comment}"
  }

  default_action {
    type                    = local.config[count.index].default_action
    forward {
        target_group {
          arn               = aws_lb_target_group.alb[count.index].arn
          weight            = 1
        }
        dynamic "stickiness" {
        for_each = { for k, v in [ local.config[count.index].stickiness ] : k => v if v != "" }
        content {
          duration          = local.config[count.index].stickiness
          enabled           = true
        }
      }
    }
  }
}

//*******************************************
// Define Application LB Target Groups
//*******************************************
resource "aws_lb_target_group" "alb" {
  count                     = length(local.config)
  
  name                      = "${local.config[count.index].env}-alb-tg-${local.config[count.index].port}"
  port                      = local.config[count.index].target_port
  protocol                  = local.config[count.index].protocol
  target_type               = local.config[count.index].target_type
  vpc_id                    = var.vpc_id

  health_check {
    enabled                 = true
    healthy_threshold       = local.config[count.index].health_checks.healthy_threshold
    interval                = local.config[count.index].health_checks.interval
    matcher                 = local.config[count.index].health_checks.success_codes
    path                    = local.config[count.index].health_checks.path
    protocol                = local.config[count.index].protocol
    timeout                 = local.config[count.index].health_checks.timeout
    unhealthy_threshold     = local.config[count.index].health_checks.unhealthy_threshold
  }

  tags = {
    Name                    = "${local.config[count.index].env}-alb-tg-${local.config[count.index].port}"
    Description             = "${local.config[count.index]._comment}"
  }

  dynamic "stickiness" {
    for_each = { for k, v in [ local.config[count.index].stickiness ] : k => v if v != "" }
    content {
      cookie_duration    = local.config[count.index].stickiness
      cookie_name           = "JSESSIONID"
      enabled               = true
      type                  = "app_cookie"
    }
  }
}

//*******************************************
// Attach ASG to Target Group
//*******************************************
resource "aws_autoscaling_attachment" "asg_attachment" {
  for_each = { for k, v in local.config[*] : k => v if v.target_type == "instance" }

  autoscaling_group_name    = local.config[index(local.config[*], each.value)].target
  lb_target_group_arn       = aws_lb_target_group.alb[index(local.config[*], each.value)].arn
}
